// kio_z3950r.cpp
//
// Author:    Robby Stephenson
// Email:     robby@periapsis.org
// Copyright: 2003
// License:   GNU General Public License, version 2

// Requires the yaz and yaz++ libraries, http://www.indexdata.dk/yaz/

#include "kio_z3950r.h"

#include <kdebug.h>
#include <kmessagebox.h>
#include <kinstance.h>
#include <klocale.h>

#include <qstring.h>

#include <yaz++/zoom.h>

using namespace KIO;

extern "C" {
  int kdemain(int argc, char** argv) {
    KInstance instance("kio_z3950r");

    kdDebug(7000) << "*** Starting kio_z3950r " << endl;

    if(argc != 4) {
      kdDebug() << "Usage: kio_z3950r protocol domain-socket1 domain-socket2" << endl;
      exit(-1);
    }

    Z3950rProtocol slave(argv[2], argv[3]);
    slave.dispatchLoop();

    kdDebug(7000) << "*** kio_z3950r Done" << endl;
    return 0;
  }
}

static const unsigned defaultPort = 210;
static const QString defaultElementSetName = QString::fromLatin1("b");;
static const QString defaultPreferredRecordSyntax = QString::fromLatin1("GRS-1");
static const unsigned defaultMaxRecords = 5000;
static const bool defaultEncodeHTML = true;

Z3950rProtocol::Z3950rProtocol(const QCString& pool_, const QCString& app_)
  : SlaveBase("z3950r", pool_, app_) {
    kdDebug(7000) << "kio_Z3950rProtocol::kio_Z3950rProtocol()" << endl;
}

void Z3950rProtocol::get(const KURL& url_) {
  kdDebug(7000) << "kio_z3950r::get()" << url_.url() << endl;

//  KURL url("z3950r://z3950.loc.gov:7090/Voyager/");
//  KURL url("z3950r://clas.caltech.edu/INNOPAC/");
//  KURL url("z3950r://z3950.bibsys.no:2100/BIBSYS/");

  KURL url = url_;
  // default port is 210
  uint port =  (url.port() > 0) ? url.port() : defaultPort;

  // there may be no query at all (empty), or an empty query ("?")
  if(url.query().isEmpty() || url.query() == "?") {
    errorPage();
    data(i18n("ERROR: Z3950r URL must include a query.").utf8());
    finished();
    return;
  }

  if(!url.hasHost()) {
    errorPage();
    data(i18n("ERROR: Z3950r URL must include a host.").utf8());
    finished();
    return;
  }

  QString hostname = url.host();
  QString dbname = url.path().section('/', 1, 1);

  if(dbname.isEmpty()) {
    errorPage();
    data(i18n("ERROR: Z3950r URL must include the database name.").utf8());
    finished();
    return;
  }

  // knock off the "?" at the beginning and decode
  // this is just used for debugging at the moment
  QString query = KURL::decode_string(url.query().mid(1));

  // the last part of the path must be search or scan (treated equally for now)
  QString tmp = url.path().section('/', -1, -1);
  if(! (tmp.lower() == "search" || tmp.lower() == "scan")) {
    errorPage();
    data(i18n("ERROR: Z3950r URL must include 'search' or 'scan'.").utf8());
    finished();
    return;
  }

  QString pqn = KURL::decode_string(url.queryItem("query"));
  // the proposed extension uses parentheses around the pqn query
  // go ahead and account for that, but don't require it
  if(pqn.startsWith("(")) {
    pqn = pqn.mid(1);
  }
  if(pqn.endsWith(")")) {
    pqn.truncate(pqn.length()-1);
  }
  
  QString esn = url.queryItem("esn");
  if(esn.isEmpty()) {
    esn = defaultElementSetName;
  }
  
  QString rs = url.queryItem("rs");
  if(rs.isEmpty()) {
    rs = defaultPreferredRecordSyntax;
  }

  bool ok;
  unsigned max = url.queryItem("maxrecs").toInt(&ok);
  if(!ok) {
    max = defaultMaxRecords;
  }

  bool encode = (url.queryItem("encodehtml") == "0") ? false : defaultEncodeHTML;

  QString msg;
  msg += "URL: "                   + url.prettyURL() + "\n";
  msg += "Hostname: "              + hostname + "\n";
  msg += "Port: "                  + QString::number(port) + "\n";
  msg += "Database: "              + dbname + "\n";
  msg += "Query: "                 + query + "\n";
  msg += "prefixQuery: "           + pqn + "\n";
  msg += "elementSetName: "        + esn + "\n";
  msg += "preferredRecordSyntax: " + rs + "\n";
  msg += "maxrecords: "            + QString::number(max) + "\n";
  msg += "encodehtml: "            + QString::number(encode) + "\n";
  
  kdDebug(7000) << msg << endl;

  infoMessage(i18n("Connecting to host..."));
  ZOOM::connection conn(hostname.latin1(), port);
  conn.option("databaseName", dbname.utf8());
  conn.option("implementationName", "kio_z3950r");
  conn.option("elementSetName", esn.utf8());
  conn.option("preferredRecordSyntax", rs.utf8());
  conn.option("count", QString::number(max).utf8());
  
  ZOOM::prefixQuery pq(pqn.utf8());
  infoMessage(i18n("Searching host..."));
  ZOOM::resultSet set(conn, pq);

//  ZOOM::connection conn("z3950.loc.gov", 7090);
//  conn.option("databaseName", "Voyager");
//  ZOOM::resultSet set(conn, ZOOM::prefixQuery("@attr 1=7 0253333490"));

  msg = i18n("Found %1 record(s).").arg(set.size());
  infoMessage(msg);
  
//  mimeType("text/plain");
  mimeType("text/html");
  
  data(QCString("<html><body>"));
//  data(QCString("<h1>")+msg.utf8()+QCString("</h1><hr>"));
  
  for(unsigned i = 0; i < QMIN(set.size(), max); ++i) {
    data(QCString("<hr><pre>"));
    QCString str(set.getRecord(i)->render());
    if(encode) {
      str = KURL::encode_string(str);
    }
    data(str);
    data(QCString("</pre>"));
  }
  data(QCString("<hr>"));
  data(QCString("</body></html>"));
  
  finished();
}
